@@ -38,13 +38,13 @@ from mirakuru.base_env import processes_with_env
 from mirakuru.exceptions import (
     AlreadyRunning,
     ProcessExitedWithError,
+    ProcessFinishedWithError,
     TimeoutExpired,
 )
 from mirakuru.compat import SIGKILL
 
 log = logging.getLogger(__name__)  # pylint: disable=invalid-name
 
-
 ENV_UUID = 'mirakuru_uuid'
 """
 Name of the environment variable used by mirakuru to mark its subprocesses.
@@ -63,13 +63,14 @@ ExecutorType = TypeVar("ExecutorType", bound="Executor")
 @atexit.register
 def cleanup_subprocesses() -> None:
     """On python exit: find possibly running subprocesses and kill them."""
-    # pylint: disable=redefined-outer-name, reimported
+    # pylint: disable=redefined-outer-name, reimported, import-outside-toplevel
     # atexit functions tends to loose global imports sometimes so reimport
     # everything what is needed again here:
     import os
     import errno
     from mirakuru.base_env import processes_with_env
     from mirakuru.compat import SIGKILL
+    # pylint: enable=redefined-outer-name, reimported, import-outside-toplevel
 
     pids = processes_with_env(ENV_UUID, str(os.getpid()))
     for pid in pids:
@@ -92,6 +93,7 @@ class SimpleExecutor:  # pylint:disable=too-many-instance-attributes
             sleep: float = 0.1,
             sig_stop: int = signal.SIGTERM,
             sig_kill: int = SIGKILL,
+            exp_sig: int = None,
             envvars: Optional[Dict[str, str]] = None,
             stdin: Union[None, int, IO[Any]] = subprocess.PIPE,
             stdout: Union[None, int, IO[Any]] = subprocess.PIPE,
@@ -111,6 +113,8 @@ class SimpleExecutor:  # pylint:disable=too-many-instance-attributes
             default is `signal.SIGTERM`
         :param int sig_kill: signal used to kill process run by the executor.
             default is `signal.SIGKILL` (`signal.SIGTERM` on Windows)
+        :param int exp_sig: expected exit code.
+            default is None as it will fall back to the stop signal
         :param dict envvars: Additional environment variables
         :param int stdin: file descriptor for stdin
         :param int stdout: file descriptor for stdout
@@ -145,6 +149,7 @@ class SimpleExecutor:  # pylint:disable=too-many-instance-attributes
         self._sleep = sleep
         self._sig_stop = sig_stop
         self._sig_kill = sig_kill
+        self._exp_sig = exp_sig
         self._envvars = envvars or {}
 
         self._stdin = stdin
@@ -294,7 +299,11 @@ class SimpleExecutor:  # pylint:disable=too-many-instance-attributes
             log.debug("Killed process %d.", pid)
         return pids
 
-    def stop(self: SimpleExecutorType, sig: int = None) -> SimpleExecutorType:
+    def stop(
+            self: SimpleExecutorType,
+            sig: int = None,
+            exp_sig: int = None
+    ) -> SimpleExecutorType:
         """
         Stop process running.
 
@@ -302,6 +311,8 @@ class SimpleExecutor:  # pylint:disable=too-many-instance-attributes
 
         :param int sig: signal used to stop process run by executor.
             None for default.
+        :param int exp_sig: expected exit code.
+            None for default.
         :returns: itself
         :rtype: SimpleExecutor
 
@@ -316,6 +327,9 @@ class SimpleExecutor:  # pylint:disable=too-many-instance-attributes
         if sig is None:
             sig = self._sig_stop
 
+        if exp_sig is None:
+            exp_sig = self._exp_sig
+
         try:
             os.killpg(self.process.pid, sig)
         except OSError as err:
@@ -335,8 +349,25 @@ class SimpleExecutor:  # pylint:disable=too-many-instance-attributes
             # at this moment, process got killed,
             pass
 
+        if self.process is None:
+            # the process has already been force killed and cleaned up by the
+            # `wait_for` above.
+            return self
         self._kill_all_kids(sig)
+        exit_code = self.process.wait()
         self._clear_process()
+
+        # Did the process shut down cleanly? A an exit code of `-sig` means
+        # that it has terminated due to signal `sig`, which is intended. So
+        # don't treat that as an error.
+        # pylint: disable=invalid-unary-operand-type
+        expected_exit_code = -sig
+        if exp_sig is not None:
+            expected_exit_code = -exp_sig
+
+        if exit_code and exit_code != expected_exit_code:
+            raise ProcessFinishedWithError(self, exit_code)
+
         return self
 
     @contextmanager
